@@ -173,12 +173,18 @@ static inline Sint32 Texture_GetBlockWidth(
     case SDL_GPU_TEXTUREFORMAT_R16_UINT:
     case SDL_GPU_TEXTUREFORMAT_R16G16_UINT:
     case SDL_GPU_TEXTUREFORMAT_R16G16B16A16_UINT:
+    case SDL_GPU_TEXTUREFORMAT_R32_UINT:
+    case SDL_GPU_TEXTUREFORMAT_R32G32_UINT:
+    case SDL_GPU_TEXTUREFORMAT_R32G32B32A32_UINT:
     case SDL_GPU_TEXTUREFORMAT_R8_INT:
     case SDL_GPU_TEXTUREFORMAT_R8G8_INT:
     case SDL_GPU_TEXTUREFORMAT_R8G8B8A8_INT:
     case SDL_GPU_TEXTUREFORMAT_R16_INT:
     case SDL_GPU_TEXTUREFORMAT_R16G16_INT:
     case SDL_GPU_TEXTUREFORMAT_R16G16B16A16_INT:
+    case SDL_GPU_TEXTUREFORMAT_R32_INT:
+    case SDL_GPU_TEXTUREFORMAT_R32G32_INT:
+    case SDL_GPU_TEXTUREFORMAT_R32G32B32A32_INT:
     case SDL_GPU_TEXTUREFORMAT_R8G8B8A8_UNORM_SRGB:
     case SDL_GPU_TEXTUREFORMAT_B8G8R8A8_UNORM_SRGB:
     case SDL_GPU_TEXTUREFORMAT_D16_UNORM:
@@ -287,12 +293,18 @@ static inline Sint32 Texture_GetBlockHeight(
     case SDL_GPU_TEXTUREFORMAT_R16_UINT:
     case SDL_GPU_TEXTUREFORMAT_R16G16_UINT:
     case SDL_GPU_TEXTUREFORMAT_R16G16B16A16_UINT:
+    case SDL_GPU_TEXTUREFORMAT_R32_UINT:
+    case SDL_GPU_TEXTUREFORMAT_R32G32_UINT:
+    case SDL_GPU_TEXTUREFORMAT_R32G32B32A32_UINT:
     case SDL_GPU_TEXTUREFORMAT_R8_INT:
     case SDL_GPU_TEXTUREFORMAT_R8G8_INT:
     case SDL_GPU_TEXTUREFORMAT_R8G8B8A8_INT:
     case SDL_GPU_TEXTUREFORMAT_R16_INT:
     case SDL_GPU_TEXTUREFORMAT_R16G16_INT:
     case SDL_GPU_TEXTUREFORMAT_R16G16B16A16_INT:
+    case SDL_GPU_TEXTUREFORMAT_R32_INT:
+    case SDL_GPU_TEXTUREFORMAT_R32G32_INT:
+    case SDL_GPU_TEXTUREFORMAT_R32G32B32A32_INT:
     case SDL_GPU_TEXTUREFORMAT_R8G8B8A8_UNORM_SRGB:
     case SDL_GPU_TEXTUREFORMAT_B8G8R8A8_UNORM_SRGB:
     case SDL_GPU_TEXTUREFORMAT_D16_UNORM:
@@ -474,12 +486,14 @@ struct SDL_GPUDevice
     SDL_GPUBuffer *(*CreateBuffer)(
         SDL_GPURenderer *driverData,
         SDL_GPUBufferUsageFlags usageFlags,
-        Uint32 size);
+        Uint32 size,
+        const char *debugName);
 
     SDL_GPUTransferBuffer *(*CreateTransferBuffer)(
         SDL_GPURenderer *driverData,
         SDL_GPUTransferBufferUsage usage,
-        Uint32 size);
+        Uint32 size,
+        const char *debugName);
 
     // Debug Naming
 
@@ -975,7 +989,7 @@ extern "C" {
 extern SDL_GPUBootstrap VulkanDriver;
 extern SDL_GPUBootstrap D3D12Driver;
 extern SDL_GPUBootstrap MetalDriver;
-extern SDL_GPUBootstrap PS5Driver;
+extern SDL_GPUBootstrap PrivateGPUDriver;
 
 #ifdef __cplusplus
 }
